@@ -1,77 +1,70 @@
 from __future__ import annotations
 
 import warnings
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Callable
-from typing import Literal
-from typing import Sequence
+from typing import TYPE_CHECKING, Any, Callable, Literal
 
-from narwhals._compliant import LazyExpr
-from narwhals._compliant.expr import DepthTrackingExpr
+from narwhals._compliant import DepthTrackingExpr, LazyExpr
 from narwhals._dask.expr_dt import DaskExprDateTimeNamespace
 from narwhals._dask.expr_str import DaskExprStringNamespace
-from narwhals._dask.utils import add_row_index
-from narwhals._dask.utils import maybe_evaluate_expr
-from narwhals._dask.utils import narwhals_to_native_dtype
-from narwhals._expression_parsing import ExprKind
-from narwhals._expression_parsing import evaluate_output_names_and_aliases
+from narwhals._dask.utils import (
+    add_row_index,
+    maybe_evaluate_expr,
+    narwhals_to_native_dtype,
+)
+from narwhals._expression_parsing import ExprKind, evaluate_output_names_and_aliases
 from narwhals._pandas_like.utils import native_to_narwhals_dtype
-from narwhals.exceptions import ColumnNotFoundError
+from narwhals._utils import (
+    Implementation,
+    generate_temporary_column_name,
+    not_implemented,
+)
 from narwhals.exceptions import InvalidOperationError
-from narwhals.utils import Implementation
-from narwhals.utils import generate_temporary_column_name
-from narwhals.utils import not_implemented
 
 if TYPE_CHECKING:
+    from collections.abc import Sequence
+
     import dask.dataframe.dask_expr as dx
     from typing_extensions import Self
 
-    from narwhals._compliant.typing import AliasNames
-    from narwhals._compliant.typing import EvalNames
-    from narwhals._compliant.typing import EvalSeries
+    from narwhals._compliant.typing import AliasNames, EvalNames, EvalSeries, ScalarKwargs
     from narwhals._dask.dataframe import DaskLazyFrame
     from narwhals._dask.namespace import DaskNamespace
-    from narwhals._expression_parsing import ExprKind
-    from narwhals._expression_parsing import ExprMetadata
-    from narwhals.dtypes import DType
-    from narwhals.typing import FillNullStrategy
-    from narwhals.typing import NonNestedLiteral
-    from narwhals.typing import NumericLiteral
-    from narwhals.typing import RollingInterpolationMethod
-    from narwhals.typing import TemporalLiteral
-    from narwhals.utils import Version
-    from narwhals.utils import _FullContext
+    from narwhals._expression_parsing import ExprKind, ExprMetadata
+    from narwhals._utils import Version, _LimitedContext
+    from narwhals.typing import (
+        FillNullStrategy,
+        IntoDType,
+        NonNestedLiteral,
+        NumericLiteral,
+        RollingInterpolationMethod,
+        TemporalLiteral,
+    )
 
 
 class DaskExpr(
-    LazyExpr["DaskLazyFrame", "dx.Series"],
-    DepthTrackingExpr["DaskLazyFrame", "dx.Series"],
+    LazyExpr["DaskLazyFrame", "dx.Series"],  # pyright: ignore[reportInvalidTypeArguments]
+    DepthTrackingExpr["DaskLazyFrame", "dx.Series"],  # pyright: ignore[reportInvalidTypeArguments]
 ):
     _implementation: Implementation = Implementation.DASK
 
     def __init__(
         self,
-        call: EvalSeries[DaskLazyFrame, dx.Series],
+        call: EvalSeries[DaskLazyFrame, dx.Series],  # pyright: ignore[reportInvalidTypeForm]
         *,
         depth: int,
         function_name: str,
         evaluate_output_names: EvalNames[DaskLazyFrame],
         alias_output_names: AliasNames | None,
-        backend_version: tuple[int, ...],
         version: Version,
-        # Kwargs with metadata which we may need in group-by agg
-        # (e.g. `ddof` for `std` and `var`).
-        call_kwargs: dict[str, Any] | None = None,
+        scalar_kwargs: ScalarKwargs | None = None,
     ) -> None:
         self._call = call
         self._depth = depth
         self._function_name = function_name
         self._evaluate_output_names = evaluate_output_names
         self._alias_output_names = alias_output_names
-        self._backend_version = backend_version
         self._version = version
-        self._call_kwargs = call_kwargs or {}
+        self._scalar_kwargs = scalar_kwargs or {}
         self._metadata: ExprMetadata | None = None
 
     def __call__(self, df: DaskLazyFrame) -> Sequence[dx.Series]:
@@ -80,14 +73,15 @@ class DaskExpr(
     def __narwhals_expr__(self) -> None: ...
 
     def __narwhals_namespace__(self) -> DaskNamespace:  # pragma: no cover
-        # Unused, just for compatibility with PandasLikeExpr
         from narwhals._dask.namespace import DaskNamespace
 
-        return DaskNamespace(backend_version=self._backend_version, version=self._version)
+        return DaskNamespace(version=self._version)
 
     def broadcast(self, kind: Literal[ExprKind.AGGREGATION, ExprKind.LITERAL]) -> Self:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
-            return [result[0] for result in self(df)]
+            # result.loc[0][0] is a workaround for dask~<=2024.10.0/dask_expr~<=1.1.16
+            #   that raised a KeyError for result[0] during collection.
+            return [result.loc[0][0] for result in self(df)]
 
         return self.__class__(
             func,
@@ -95,9 +89,8 @@ class DaskExpr(
             function_name=self._function_name,
             evaluate_output_names=self._evaluate_output_names,
             alias_output_names=self._alias_output_names,
-            backend_version=self._backend_version,
             version=self._version,
-            call_kwargs=self._call_kwargs,
+            scalar_kwargs=self._scalar_kwargs,
         )
 
     @classmethod
@@ -106,7 +99,7 @@ class DaskExpr(
         evaluate_column_names: EvalNames[DaskLazyFrame],
         /,
         *,
-        context: _FullContext,
+        context: _LimitedContext,
         function_name: str = "",
     ) -> Self:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
@@ -116,13 +109,9 @@ class DaskExpr(
                     for column_name in evaluate_column_names(df)
                 ]
             except KeyError as e:
-                missing_columns = [
-                    x for x in evaluate_column_names(df) if x not in df.columns
-                ]
-                raise ColumnNotFoundError.from_missing_and_available_column_names(
-                    missing_columns=missing_columns,
-                    available_columns=df.columns,
-                ) from e
+                if error := df._check_columns_exist(evaluate_column_names(df)):
+                    raise error from e
+                raise
 
         return cls(
             func,
@@ -130,26 +119,20 @@ class DaskExpr(
             function_name=function_name,
             evaluate_output_names=evaluate_column_names,
             alias_output_names=None,
-            backend_version=context._backend_version,
             version=context._version,
         )
 
     @classmethod
-    def from_column_indices(
-        cls: type[Self], *column_indices: int, context: _FullContext
-    ) -> Self:
+    def from_column_indices(cls, *column_indices: int, context: _LimitedContext) -> Self:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
-            return [
-                df._native_frame.iloc[:, column_index] for column_index in column_indices
-            ]
+            return [df.native.iloc[:, i] for i in column_indices]
 
         return cls(
             func,
             depth=0,
             function_name="nth",
-            evaluate_output_names=lambda df: [df.columns[i] for i in column_indices],
+            evaluate_output_names=cls._eval_names_indices(column_indices),
             alias_output_names=None,
-            backend_version=context._backend_version,
             version=context._version,
         )
 
@@ -159,7 +142,7 @@ class DaskExpr(
         call: Callable[..., dx.Series],
         /,
         expr_name: str = "",
-        call_kwargs: dict[str, Any] | None = None,
+        scalar_kwargs: ScalarKwargs | None = None,
         **expressifiable_args: Self | Any,
     ) -> Self:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
@@ -180,128 +163,134 @@ class DaskExpr(
             function_name=f"{self._function_name}->{expr_name}",
             evaluate_output_names=self._evaluate_output_names,
             alias_output_names=self._alias_output_names,
-            backend_version=self._backend_version,
             version=self._version,
-            call_kwargs=call_kwargs,
+            scalar_kwargs=scalar_kwargs,
         )
 
     def _with_alias_output_names(self, func: AliasNames | None, /) -> Self:
+        current_alias_output_names = self._alias_output_names
+        alias_output_names = (
+            None
+            if func is None
+            else func
+            if current_alias_output_names is None
+            else lambda output_names: func(current_alias_output_names(output_names))
+        )
         return type(self)(
             call=self._call,
             depth=self._depth,
             function_name=self._function_name,
             evaluate_output_names=self._evaluate_output_names,
-            alias_output_names=func,
-            backend_version=self._backend_version,
+            alias_output_names=alias_output_names,
             version=self._version,
-            call_kwargs=self._call_kwargs,
+            scalar_kwargs=self._scalar_kwargs,
         )
 
     def __add__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__add__(other), "__add__", other=other
+            lambda expr, other: expr.__add__(other), "__add__", other=other
         )
 
     def __sub__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__sub__(other), "__sub__", other=other
+            lambda expr, other: expr.__sub__(other), "__sub__", other=other
         )
 
     def __rsub__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: other - _input, "__rsub__", other=other
+            lambda expr, other: other - expr, "__rsub__", other=other
         ).alias("literal")
 
     def __mul__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__mul__(other), "__mul__", other=other
+            lambda expr, other: expr.__mul__(other), "__mul__", other=other
         )
 
     def __truediv__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__truediv__(other), "__truediv__", other=other
+            lambda expr, other: expr.__truediv__(other), "__truediv__", other=other
         )
 
     def __rtruediv__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: other / _input, "__rtruediv__", other=other
+            lambda expr, other: other / expr, "__rtruediv__", other=other
         ).alias("literal")
 
     def __floordiv__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__floordiv__(other), "__floordiv__", other=other
+            lambda expr, other: expr.__floordiv__(other), "__floordiv__", other=other
         )
 
     def __rfloordiv__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: other // _input, "__rfloordiv__", other=other
+            lambda expr, other: other // expr, "__rfloordiv__", other=other
         ).alias("literal")
 
     def __pow__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__pow__(other), "__pow__", other=other
+            lambda expr, other: expr.__pow__(other), "__pow__", other=other
         )
 
     def __rpow__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: other**_input, "__rpow__", other=other
+            lambda expr, other: other**expr, "__rpow__", other=other
         ).alias("literal")
 
     def __mod__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__mod__(other), "__mod__", other=other
+            lambda expr, other: expr.__mod__(other), "__mod__", other=other
         )
 
     def __rmod__(self, other: Any) -> Self:
         return self._with_callable(
-            lambda _input, other: other % _input, "__rmod__", other=other
+            lambda expr, other: other % expr, "__rmod__", other=other
         ).alias("literal")
 
     def __eq__(self, other: DaskExpr) -> Self:  # type: ignore[override]
         return self._with_callable(
-            lambda _input, other: _input.__eq__(other), "__eq__", other=other
+            lambda expr, other: expr.__eq__(other), "__eq__", other=other
         )
 
     def __ne__(self, other: DaskExpr) -> Self:  # type: ignore[override]
         return self._with_callable(
-            lambda _input, other: _input.__ne__(other), "__ne__", other=other
+            lambda expr, other: expr.__ne__(other), "__ne__", other=other
         )
 
     def __ge__(self, other: DaskExpr | Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__ge__(other), "__ge__", other=other
+            lambda expr, other: expr.__ge__(other), "__ge__", other=other
         )
 
     def __gt__(self, other: DaskExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__gt__(other), "__gt__", other=other
+            lambda expr, other: expr.__gt__(other), "__gt__", other=other
         )
 
     def __le__(self, other: DaskExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__le__(other), "__le__", other=other
+            lambda expr, other: expr.__le__(other), "__le__", other=other
         )
 
     def __lt__(self, other: DaskExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__lt__(other), "__lt__", other=other
+            lambda expr, other: expr.__lt__(other), "__lt__", other=other
         )
 
     def __and__(self, other: DaskExpr | Any) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__and__(other), "__and__", other=other
+            lambda expr, other: expr.__and__(other), "__and__", other=other
         )
 
     def __or__(self, other: DaskExpr) -> Self:
         return self._with_callable(
-            lambda _input, other: _input.__or__(other), "__or__", other=other
+            lambda expr, other: expr.__or__(other), "__or__", other=other
         )
 
     def __invert__(self) -> Self:
-        return self._with_callable(lambda _input: _input.__invert__(), "__invert__")
+        return self._with_callable(lambda expr: expr.__invert__(), "__invert__")
 
     def mean(self) -> Self:
-        return self._with_callable(lambda _input: _input.mean().to_series(), "mean")
+        return self._with_callable(lambda expr: expr.mean().to_series(), "mean")
 
     def median(self) -> Self:
         from narwhals.exceptions import InvalidOperationError
@@ -316,30 +305,33 @@ class DaskExpr(
         return self._with_callable(func, "median")
 
     def min(self) -> Self:
-        return self._with_callable(lambda _input: _input.min().to_series(), "min")
+        return self._with_callable(lambda expr: expr.min().to_series(), "min")
 
     def max(self) -> Self:
-        return self._with_callable(lambda _input: _input.max().to_series(), "max")
+        return self._with_callable(lambda expr: expr.max().to_series(), "max")
 
     def std(self, ddof: int) -> Self:
         return self._with_callable(
-            lambda _input: _input.std(ddof=ddof).to_series(),
+            lambda expr: expr.std(ddof=ddof).to_series(),
             "std",
-            call_kwargs={"ddof": ddof},
+            scalar_kwargs={"ddof": ddof},
         )
 
     def var(self, ddof: int) -> Self:
         return self._with_callable(
-            lambda _input: _input.var(ddof=ddof).to_series(),
+            lambda expr: expr.var(ddof=ddof).to_series(),
             "var",
-            call_kwargs={"ddof": ddof},
+            scalar_kwargs={"ddof": ddof},
         )
 
     def skew(self) -> Self:
-        return self._with_callable(lambda _input: _input.skew().to_series(), "skew")
+        return self._with_callable(lambda expr: expr.skew().to_series(), "skew")
+
+    def kurtosis(self) -> Self:
+        return self._with_callable(lambda expr: expr.kurtosis().to_series(), "kurtosis")
 
     def shift(self, n: int) -> Self:
-        return self._with_callable(lambda _input: _input.shift(n), "shift")
+        return self._with_callable(lambda expr: expr.shift(n), "shift")
 
     def cum_sum(self, *, reverse: bool) -> Self:
         if reverse:  # pragma: no cover
@@ -347,7 +339,7 @@ class DaskExpr(
             msg = "`cum_sum(reverse=True)` is not supported with Dask backend"
             raise NotImplementedError(msg)
 
-        return self._with_callable(lambda _input: _input.cumsum(), "cum_sum")
+        return self._with_callable(lambda expr: expr.cumsum(), "cum_sum")
 
     def cum_count(self, *, reverse: bool) -> Self:
         if reverse:  # pragma: no cover
@@ -355,7 +347,7 @@ class DaskExpr(
             raise NotImplementedError(msg)
 
         return self._with_callable(
-            lambda _input: (~_input.isna()).astype(int).cumsum(), "cum_count"
+            lambda expr: (~expr.isna()).astype(int).cumsum(), "cum_count"
         )
 
     def cum_min(self, *, reverse: bool) -> Self:
@@ -363,25 +355,25 @@ class DaskExpr(
             msg = "`cum_min(reverse=True)` is not supported with Dask backend"
             raise NotImplementedError(msg)
 
-        return self._with_callable(lambda _input: _input.cummin(), "cum_min")
+        return self._with_callable(lambda expr: expr.cummin(), "cum_min")
 
     def cum_max(self, *, reverse: bool) -> Self:
         if reverse:  # pragma: no cover
             msg = "`cum_max(reverse=True)` is not supported with Dask backend"
             raise NotImplementedError(msg)
 
-        return self._with_callable(lambda _input: _input.cummax(), "cum_max")
+        return self._with_callable(lambda expr: expr.cummax(), "cum_max")
 
     def cum_prod(self, *, reverse: bool) -> Self:
         if reverse:  # pragma: no cover
             msg = "`cum_prod(reverse=True)` is not supported with Dask backend"
             raise NotImplementedError(msg)
 
-        return self._with_callable(lambda _input: _input.cumprod(), "cum_prod")
+        return self._with_callable(lambda expr: expr.cumprod(), "cum_prod")
 
     def rolling_sum(self, window_size: int, *, min_samples: int, center: bool) -> Self:
         return self._with_callable(
-            lambda _input: _input.rolling(
+            lambda expr: expr.rolling(
                 window=window_size, min_periods=min_samples, center=center
             ).sum(),
             "rolling_sum",
@@ -389,7 +381,7 @@ class DaskExpr(
 
     def rolling_mean(self, window_size: int, *, min_samples: int, center: bool) -> Self:
         return self._with_callable(
-            lambda _input: _input.rolling(
+            lambda expr: expr.rolling(
                 window=window_size, min_periods=min_samples, center=center
             ).mean(),
             "rolling_mean",
@@ -400,50 +392,48 @@ class DaskExpr(
     ) -> Self:
         if ddof == 1:
             return self._with_callable(
-                lambda _input: _input.rolling(
+                lambda expr: expr.rolling(
                     window=window_size, min_periods=min_samples, center=center
                 ).var(),
                 "rolling_var",
             )
-        else:
-            msg = "Dask backend only supports `ddof=1` for `rolling_var`"
-            raise NotImplementedError(msg)
+        msg = "Dask backend only supports `ddof=1` for `rolling_var`"
+        raise NotImplementedError(msg)
 
     def rolling_std(
         self, window_size: int, *, min_samples: int, center: bool, ddof: int
     ) -> Self:
         if ddof == 1:
             return self._with_callable(
-                lambda _input: _input.rolling(
+                lambda expr: expr.rolling(
                     window=window_size, min_periods=min_samples, center=center
                 ).std(),
                 "rolling_std",
             )
-        else:
-            msg = "Dask backend only supports `ddof=1` for `rolling_std`"
-            raise NotImplementedError(msg)
+        msg = "Dask backend only supports `ddof=1` for `rolling_std`"
+        raise NotImplementedError(msg)
 
     def sum(self) -> Self:
-        return self._with_callable(lambda _input: _input.sum().to_series(), "sum")
+        return self._with_callable(lambda expr: expr.sum().to_series(), "sum")
 
     def count(self) -> Self:
-        return self._with_callable(lambda _input: _input.count().to_series(), "count")
+        return self._with_callable(lambda expr: expr.count().to_series(), "count")
 
     def round(self, decimals: int) -> Self:
-        return self._with_callable(lambda _input: _input.round(decimals), "round")
+        return self._with_callable(lambda expr: expr.round(decimals), "round")
 
     def unique(self) -> Self:
-        return self._with_callable(lambda _input: _input.unique(), "unique")
+        return self._with_callable(lambda expr: expr.unique(), "unique")
 
     def drop_nulls(self) -> Self:
-        return self._with_callable(lambda _input: _input.dropna(), "drop_nulls")
+        return self._with_callable(lambda expr: expr.dropna(), "drop_nulls")
 
     def abs(self) -> Self:
-        return self._with_callable(lambda _input: _input.abs(), "abs")
+        return self._with_callable(lambda expr: expr.abs(), "abs")
 
     def all(self) -> Self:
         return self._with_callable(
-            lambda _input: _input.all(
+            lambda expr: expr.all(
                 axis=None, skipna=True, split_every=False, out=None
             ).to_series(),
             "all",
@@ -451,7 +441,7 @@ class DaskExpr(
 
     def any(self) -> Self:
         return self._with_callable(
-            lambda _input: _input.any(axis=0, skipna=True, split_every=False).to_series(),
+            lambda expr: expr.any(axis=0, skipna=True, split_every=False).to_series(),
             "any",
         )
 
@@ -461,14 +451,14 @@ class DaskExpr(
         strategy: FillNullStrategy | None,
         limit: int | None,
     ) -> Self:
-        def func(_input: dx.Series) -> dx.Series:
+        def func(expr: dx.Series) -> dx.Series:
             if value is not None:
-                res_ser = _input.fillna(value)
+                res_ser = expr.fillna(value)
             else:
                 res_ser = (
-                    _input.ffill(limit=limit)
+                    expr.ffill(limit=limit)
                     if strategy == "forward"
-                    else _input.bfill(limit=limit)
+                    else expr.bfill(limit=limit)
                 )
             return res_ser
 
@@ -480,7 +470,7 @@ class DaskExpr(
         upper_bound: Self | NumericLiteral | TemporalLiteral | None,
     ) -> Self:
         return self._with_callable(
-            lambda _input, lower_bound, upper_bound: _input.clip(
+            lambda expr, lower_bound, upper_bound: expr.clip(
                 lower=lower_bound, upper=upper_bound
             ),
             "clip",
@@ -489,78 +479,73 @@ class DaskExpr(
         )
 
     def diff(self) -> Self:
-        return self._with_callable(lambda _input: _input.diff(), "diff")
+        return self._with_callable(lambda expr: expr.diff(), "diff")
 
     def n_unique(self) -> Self:
         return self._with_callable(
-            lambda _input: _input.nunique(dropna=False).to_series(), "n_unique"
+            lambda expr: expr.nunique(dropna=False).to_series(), "n_unique"
         )
 
     def is_null(self) -> Self:
-        return self._with_callable(lambda _input: _input.isna(), "is_null")
+        return self._with_callable(lambda expr: expr.isna(), "is_null")
 
     def is_nan(self) -> Self:
-        def func(_input: dx.Series) -> dx.Series:
+        def func(expr: dx.Series) -> dx.Series:
             dtype = native_to_narwhals_dtype(
-                _input.dtype, self._version, self._implementation
+                expr.dtype, self._version, self._implementation
             )
             if dtype.is_numeric():
-                return _input != _input  # pyright: ignore[reportReturnType] # noqa: PLR0124
+                return expr != expr  # pyright: ignore[reportReturnType] # noqa: PLR0124
             msg = f"`.is_nan` only supported for numeric dtypes and not {dtype}, did you mean `.is_null`?"
             raise InvalidOperationError(msg)
 
         return self._with_callable(func, "is_null")
 
     def len(self) -> Self:
-        return self._with_callable(lambda _input: _input.size.to_series(), "len")
+        return self._with_callable(lambda expr: expr.size.to_series(), "len")
 
     def quantile(
         self, quantile: float, interpolation: RollingInterpolationMethod
     ) -> Self:
         if interpolation == "linear":
 
-            def func(_input: dx.Series, quantile: float) -> dx.Series:
-                if _input.npartitions > 1:
+            def func(expr: dx.Series, quantile: float) -> dx.Series:
+                if expr.npartitions > 1:
                     msg = "`Expr.quantile` is not supported for Dask backend with multiple partitions."
                     raise NotImplementedError(msg)
-                return _input.quantile(
+                return expr.quantile(
                     q=quantile, method="dask"
                 ).to_series()  # pragma: no cover
 
             return self._with_callable(func, "quantile", quantile=quantile)
-        else:
-            msg = "`higher`, `lower`, `midpoint`, `nearest` - interpolation methods are not supported by Dask. Please use `linear` instead."
-            raise NotImplementedError(msg)
+        msg = "`higher`, `lower`, `midpoint`, `nearest` - interpolation methods are not supported by Dask. Please use `linear` instead."
+        raise NotImplementedError(msg)
 
     def is_first_distinct(self) -> Self:
-        def func(_input: dx.Series) -> dx.Series:
-            _name = _input.name
+        def func(expr: dx.Series) -> dx.Series:
+            _name = expr.name
             col_token = generate_temporary_column_name(n_bytes=8, columns=[_name])
-            frame = add_row_index(
-                _input.to_frame(), col_token, self._backend_version, self._implementation
-            )
+            frame = add_row_index(expr.to_frame(), col_token)
             first_distinct_index = frame.groupby(_name).agg({col_token: "min"})[col_token]
             return frame[col_token].isin(first_distinct_index)
 
         return self._with_callable(func, "is_first_distinct")
 
     def is_last_distinct(self) -> Self:
-        def func(_input: dx.Series) -> dx.Series:
-            _name = _input.name
+        def func(expr: dx.Series) -> dx.Series:
+            _name = expr.name
             col_token = generate_temporary_column_name(n_bytes=8, columns=[_name])
-            frame = add_row_index(
-                _input.to_frame(), col_token, self._backend_version, self._implementation
-            )
+            frame = add_row_index(expr.to_frame(), col_token)
             last_distinct_index = frame.groupby(_name).agg({col_token: "max"})[col_token]
             return frame[col_token].isin(last_distinct_index)
 
         return self._with_callable(func, "is_last_distinct")
 
     def is_unique(self) -> Self:
-        def func(_input: dx.Series) -> dx.Series:
-            _name = _input.name
+        def func(expr: dx.Series) -> dx.Series:
+            _name = expr.name
             return (
-                _input.to_frame()
+                expr.to_frame()
                 .groupby(_name, dropna=False)
                 .transform("size", meta=(_name, int))
                 == 1
@@ -569,23 +554,19 @@ class DaskExpr(
         return self._with_callable(func, "is_unique")
 
     def is_in(self, other: Any) -> Self:
-        return self._with_callable(lambda _input: _input.isin(other), "is_in")
+        return self._with_callable(lambda expr: expr.isin(other), "is_in")
 
     def null_count(self) -> Self:
         return self._with_callable(
-            lambda _input: _input.isna().sum().to_series(), "null_count"
+            lambda expr: expr.isna().sum().to_series(), "null_count"
         )
 
-    def over(
-        self,
-        partition_by: Sequence[str],
-        order_by: Sequence[str] | None,
-    ) -> Self:
+    def over(self, partition_by: Sequence[str], order_by: Sequence[str]) -> Self:
         # pandas is a required dependency of dask so it's safe to import this
         from narwhals._pandas_like.group_by import PandasLikeGroupBy
 
         if not partition_by:
-            assert order_by is not None  # help type checkers  # noqa: S101
+            assert order_by  # noqa: S101
 
             # This is something like `nw.col('a').cum_sum().order_by(key)`
             # which we can always easily support, as it doesn't require grouping.
@@ -630,11 +611,11 @@ class DaskExpr(
                             msg = "Safety check failed, please report a bug."
                             raise AssertionError(msg)
                         res_native = grouped.transform(
-                            dask_function_name, **self._call_kwargs
+                            dask_function_name, **self._scalar_kwargs
                         ).to_frame(output_names[0])
                     else:
                         res_native = grouped[list(output_names)].transform(
-                            dask_function_name, **self._call_kwargs
+                            dask_function_name, **self._scalar_kwargs
                         )
                 result_frame = df._with_native(
                     res_native.rename(columns=dict(zip(output_names, aliases)))
@@ -647,14 +628,13 @@ class DaskExpr(
             function_name=self._function_name + "->over",
             evaluate_output_names=self._evaluate_output_names,
             alias_output_names=self._alias_output_names,
-            backend_version=self._backend_version,
             version=self._version,
         )
 
-    def cast(self, dtype: DType | type[DType]) -> Self:
-        def func(_input: dx.Series) -> dx.Series:
+    def cast(self, dtype: IntoDType) -> Self:
+        def func(expr: dx.Series) -> dx.Series:
             native_dtype = narwhals_to_native_dtype(dtype, self._version)
-            return _input.astype(native_dtype)
+            return expr.astype(native_dtype)
 
         return self._with_callable(func, "cast")
 
@@ -663,6 +643,24 @@ class DaskExpr(
 
         return self._with_callable(da.isfinite, "is_finite")
 
+    def log(self, base: float) -> Self:
+        import dask.array as da
+
+        def _log(expr: dx.Series) -> dx.Series:
+            return da.log(expr) / da.log(base)
+
+        return self._with_callable(_log, "log")
+
+    def exp(self) -> Self:
+        import dask.array as da
+
+        return self._with_callable(da.exp, "exp")
+
+    def sqrt(self) -> Self:
+        import dask.array as da
+
+        return self._with_callable(da.sqrt, "sqrt")
+
     @property
     def str(self) -> DaskExprStringNamespace:
         return DaskExprStringNamespace(self)
@@ -671,6 +669,21 @@ class DaskExpr(
     def dt(self) -> DaskExprDateTimeNamespace:
         return DaskExprDateTimeNamespace(self)
 
-    list = not_implemented()  # pyright: ignore[reportAssignmentType]
-    struct = not_implemented()  # pyright: ignore[reportAssignmentType]
-    rank = not_implemented()  # pyright: ignore[reportAssignmentType]
+    arg_max: not_implemented = not_implemented()
+    arg_min: not_implemented = not_implemented()
+    arg_true: not_implemented = not_implemented()
+    ewm_mean: not_implemented = not_implemented()
+    gather_every: not_implemented = not_implemented()
+    head: not_implemented = not_implemented()
+    map_batches: not_implemented = not_implemented()
+    mode: not_implemented = not_implemented()
+    sample: not_implemented = not_implemented()
+    rank: not_implemented = not_implemented()
+    replace_strict: not_implemented = not_implemented()
+    sort: not_implemented = not_implemented()
+    tail: not_implemented = not_implemented()
+
+    # namespaces
+    list: not_implemented = not_implemented()  # type: ignore[assignment]
+    cat: not_implemented = not_implemented()  # type: ignore[assignment]
+    struct: not_implemented = not_implemented()  # type: ignore[assignment]
